@@ -5,7 +5,7 @@ use urpm::msg;
 use urpm::cfg;
 use Cwd;
 
-(our $VERSION) = q$Id: download.pm,v 1.51 2006/01/27 15:52:11 rgarciasuarez Exp $ =~ /(\d+\.\d+)/;
+(our $VERSION) = q$Id: download.pm,v 1.53 2006/03/20 13:30:18 rgarciasuarez Exp $ =~ /(\d+\.\d+)/;
 
 #- proxy config file.
 our $PROXY_CFG = '/etc/urpmi/proxy.cfg';
@@ -177,11 +177,11 @@ sub set_proxy {
 }
 
 sub propagate_sync_callback {
-    my $options = shift @_;
+    my $options = shift;
     if (ref($options) && $options->{callback}) {
-	my $mode = shift @_;
+	my $mode = shift;
 	if ($mode =~ /^(?:start|progress|end)$/) {
-	    my $file = shift @_;
+	    my $file = shift;
 	    $file =~ s|([^:]*://[^/:\@]*:)[^/:\@]*(\@.*)|$1xxxx$2|; #- if needed...
 	    return $options->{callback}($mode, $file, @_);
 	} else {
@@ -204,7 +204,7 @@ sub sync_file {
 
 sub sync_wget {
     -x "/usr/bin/wget" or die N("wget is missing\n");
-    my $options = shift @_;
+    my $options = shift;
     $options = { dir => $options } if !ref $options;
     #- force download to be done in cachedir to avoid polluting cwd.
     my $cwd = getcwd();
@@ -267,7 +267,7 @@ sub sync_wget {
 
 sub sync_curl {
     -x "/usr/bin/curl" or die N("curl is missing\n");
-    my $options = shift @_;
+    my $options = shift;
     $options = { dir => $options } if !ref $options;
     if (defined $options->{limit_rate} && $options->{limit_rate} =~ /\d$/) {
 	#- use bytes by default
@@ -421,7 +421,7 @@ sub _calc_limit_rate {
 
 sub sync_rsync {
     -x "/usr/bin/rsync" or die N("rsync is missing\n");
-    my $options = shift @_;
+    my $options = shift;
     $options = { dir => $options } if !ref $options;
     #- force download to be done in cachedir to avoid polluting cwd.
     my $cwd = getcwd();
@@ -439,7 +439,7 @@ sub sync_rsync {
 		($limit_rate ? "--bwlimit=$limit_rate" : @{[]}),
 		($options->{quiet} ? qw(-q) : qw(--progress -v)),
 		($options->{compress} ? qw(-z) : @{[]}),
-		($options->{ssh} ? qw(-e ssh) : @{[]}),
+		($options->{ssh} ? qq(-e $options->{ssh}) : @{[]}),
 		qw(--partial --no-whole-file),
 		(defined $options->{'rsync-options'} ? split /\s+/, $options->{'rsync-options'} : ()),
 		"'$file' '$options->{dir}' |");
@@ -465,14 +465,63 @@ sub sync_rsync {
     $? == 0 or die N("rsync failed: exited with %d or signal %d\n", $? >> 8, $? & 127);
 }
 
+our $SSH_PATH;
+sub _init_ssh_path {
+    for (qw(/usr/bin/ssh /bin/ssh)) {
+	-x $_ and $SSH_PATH = $_;
+	next;
+    }
+}
+
+#- Don't generate a tmp dir name, so when we restart urpmi, the old ssh
+#- connection can be reused
+our $SSH_CONTROL_DIR = $ENV{TMP} || $ENV{TMPDIR} || '/tmp';
+our $SSH_CONTROL_OPTION;
+
 sub sync_ssh {
-    -x "/usr/bin/ssh" or die N("ssh is missing\n");
-    my $options = shift(@_);
+    $SSH_PATH or _init_ssh_path();
+    $SSH_PATH or die N("ssh is missing\n");
+    my $options = shift;
     $options = { dir => $options } if !ref $options;
-    $options->{ssh} = 1;
+    unless ($options->{'rsync-options'} =~ /(?:-e|--rsh)\b/) {
+	my $server = '';
+	$_[0] =~ /((?:\w|\.)*):/ and $server = $1;
+	$SSH_CONTROL_OPTION = "-o 'ControlPath $SSH_CONTROL_DIR/ssh-urpmi-$$-%h_%p_%r' -o 'ControlMaster auto'";
+	if (start_ssh_master($server)) {
+	    $options->{ssh} = qq("$SSH_PATH $SSH_CONTROL_OPTION");
+	} else {
+	    #- can't start master, use single connection
+	    $options->{ssh} = $SSH_PATH;
+	}
+    }
     sync_rsync($options, @_);
 }
 
+sub start_ssh_master {
+    my $server = shift;
+    $server or return 0;
+    if (!check_ssh_master($server)) {
+        system(qq($SSH_PATH -f -N $SSH_CONTROL_OPTION -M $server));
+	return ! $?;
+    }
+    return 1;
+}
+
+sub check_ssh_master {
+    my $server = shift;
+    system(qq($SSH_PATH -q -f -N  $SSH_CONTROL_OPTION $server -O check));
+    return ! $?;
+}
+
+END {
+    #- remove ssh persistent connections
+    for my $socket (glob "$SSH_CONTROL_DIR/ssh-urpmi-$$-*") {
+	$socket =~ /ssh-urpmi-\d+-([^_]+)_/;
+	my $server = $1 or next;
+        system("$SSH_PATH -q -f -N -o 'ControlPath $socket' -O exit $server");
+    }
+}
+
 #- get the width of the terminal
 my $wchar = 79;
 eval {
@@ -519,6 +568,6 @@ urpm::download - download routines for the urpm* tools
 
 Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005 MandrakeSoft SA
 
-Copyright (C) 2005 Mandriva SA
+Copyright (C) 2005, 2006 Mandriva SA
 
 =cut
